package LATMOS::Accounts::Web::Controller::Json::Employment;
use Moose;
use namespace::autoclean;

use DateTime;
use Date::Parse;

BEGIN { extends 'Catalyst::Controller'; }

=head1 NAME

LATMOS::Accounts::Web::Controller::Json::Employment - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;

    my $base = $c->model('Accounts')->db;

    my %employments;
    my $now = DateTime->now->ymd;

    my $key = $c->req->param('key') || 'department';
    my @filter = $c->req->param('filter');

    my $keyobj = $key =~ m/^(company|department|contratType|manager)$/
        ? 'employment'
        : 'user';

    my $filterobj = @filter[0] =~ m/^(company|department|contratType|manager)=/
        ? 'employment'
        : 'user';

    my %end = $base->attributes_summary_by_object('employment', 'lastday');
    my %start = $base->attributes_summary_by_object('employment', 'firstday');
    my %empusers = $base->attributes_summary_by_object('employment', 'user');
    my %keys = $base->attributes_summary_by_object($keyobj, $key);

    my %filteredObj = map { $_ => 1 } $base->search_objects($filterobj, @filter);

    my %keysentry;
    foreach my $emp (keys %start) {
        # We filter pasted employment
        next if ($end{$emp}[0] && $end{$emp}[0] lt $now);

        if ($filterobj eq 'user') {
            $filteredObj{ $empusers{$emp}[0] } or next;
        } else {
            $filteredObj{ $emp } or next;
        }
 
        my $keyvalues = $keyobj eq 'user'
            ? $keys{ $empusers{$emp}[0] }
            : $keys {$emp};

        $employments{ $emp } = {
           start => $start{$emp}[0],
           end   => $end  {$emp}[0],
           key   => $keyvalues,
       };
       $keysentry{ $keyvalues->[0] || $c->loc('(none)') } = [];
    }

    my %done;

    my @dates = sort
        map { $employments{$_}->{end}  }
        grep { $employments{$_}->{end} } (keys %employments);

    if (@dates) {
        my $dt = DateTime->from_epoch( epoch => str2time($dates[-1]));
        $dt->add(days => 60);
        push(@dates, $dt->ymd);
    }

    if (!@dates) {
        foreach (1 .. 12) {
            push(@dates, DateTime->now->add(months => $_)->ymd);
        }
    }

    push(@dates, $now);

    foreach my $emp (sort { $a cmp $b } @dates) {
        $done{$emp} and next;
        $done{$emp} = 1;

        my %sum = map { $_ => 0 } keys %keysentry;
        $sum{ $c->loc('(none)') } = 0;

        foreach my $cemp (keys %employments) {
            #warn "$cemp $emp" . $employments{$cemp}->{start} . " " . $employments{$cemp}->{end} ;
            if ($employments{$cemp}->{start} le $emp && ((!$employments{$cemp}->{end}) || $employments{$cemp}->{end} ge $emp)) {
                foreach my $k (@{ $employments{$cemp}->{key} || [ $c->loc('(none)') ] }) {
                    $sum{$k} ++;
                }
            }
        }

        foreach (keys %keysentry) {
           push (@{ $keysentry{$_} }, [ $emp, $sum{$_}+0 ]);
       } 
    }

    my $data;
    foreach my $k (sort { $keysentry{$a}->[0][1] <=> $keysentry{$b}->[0][1] } keys %keysentry) {
        push(@{ $data->{data} }, $keysentry{$k});
        push(@{ $data->{series} }, { label => $k || $c->loc('(none)') });
    }

    $c->stash->{data} = $data;

}






=encoding utf8

=head1 AUTHOR

Olivier Thauvin,Guyancourt - B1428,+33 1 80285052,

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
